# OpenSCAD model package makefile fragment
#
# Copyright (C) 2016, 2017, 2019  Libiquity LLC
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

OBJECTS = $(SOURCES:.scad=.stl)
DEPENDS = $(SOURCES:.scad=.d)
CONFIGS = $(SOURCES:.scad=.ini)
SCRIPTS = $(SOURCES:.scad=.gcode)

COMP_LVL_GZIP  = -9
COMP_LVL_BZIP2 = -9
COMP_LVL_XZ    = -0e

DIST_DIR = $(PACKAGE_NAME)-$(PACKAGE_VERSION)
DIST_COMMON = Makefile build.mk
DIST_FILES = $(CONFIGS) $(DIST_COMMON) $(EXTRA_DIST)
DIST_TARGETS = dist-gzip dist-bzip2 dist-xz
DIST_ARCHIVES = $(DIST_DIR).tar.gz $(DIST_DIR).tar.bz2 $(DIST_DIR).tar.xz
DIST_ARCHIVE = $(DIST_DIR).tar.$(DIST_ARCHIVE_$(@)_EXT)
DIST_ARCHIVE_dist-gzip_EXT = gz
DIST_ARCHIVE_dist-bzip2_EXT = bz2
DIST_ARCHIVE_dist-xz_EXT = xz
DO_RM_DIST_DIR =

V = 0
V_OPENSCAD = $(V_OPENSCAD_$(V))
V_OPENSCAD_0 = @printf '  OPENSCAD %s\n' $*.stl;
V_OPENSCAD_1 =
V_SLIC3R = $(V_SLIC3R_$(V))
V_SLIC3R_0 = @printf '  SLIC3R   %s\n' $*.gcode;
V_SLIC3R_1 =
V_DISTDIR = $(V_DISTDIR_$(V))
V_DISTDIR_0 = @printf '  GENERATE %s\n' $(DIST_DIR).tar;
V_DISTDIR_1 =
V_DIST = $(V_DIST_$(V))
V_DIST_0 = @printf '  COMPRESS %s\n' $(DIST_ARCHIVE);
V_DIST_1 =
Q = $(Q_$(V))
Q_0 = @
Q_1 =

# Removes absolute paths from OpenSCAD-generated dependency makefiles
DEP_AWK_SCRIPT = '\
	{ \
		if(index($$0, "\t" PWD) == 1) { \
			print("\t" substr($$0, length(PWD) + 2)); \
		} else { \
			print($$0); \
		} \
	} \
	'

OPENSCAD = openscad
SLIC3R = slic3r
MKDIR_P = mkdir -p
GIT = git
TAR = tar
GZIP = gzip
BZIP2 = bzip2
XZ = xz

BED_CENTER = 100,100

.SUFFIXES:
.SUFFIXES: .scad .stl .d .gcode

all: $(SCRIPTS)

-include $(DEPENDS)

clean:
	rm -f $(OBJECTS)

distclean: clean
	rm -f $(DEPENDS)

distdir: $(DEPENDS)
	$(V_DISTDIR)set -e; \
	rm -Rf $(DIST_DIR); \
	for f in $(DIST_FILES); do \
		f=./$${f}; \
		$(MKDIR_P) $(DIST_DIR)/$${f%/*}; \
		cp -p $${f} $(DIST_DIR)/$${f%/*}; \
	done; \
	for d in $(DEPENDS); do \
		f=$${d%.d}.scad; \
		p=$$(sed -n "\$$s|\t\(.*\)$${f}|\1|p" $${d}); \
		tail -n +2 $${d} | while read -r l; do \
			l=$${l% \\}; \
			case $${l} in \
				$${p}*) \
					l=$${l#$${p}}; \
					$(MKDIR_P) $(DIST_DIR)/$${l%/*}; \
					cp -p $${l} $(DIST_DIR)/$${l%/*}; \
					;; \
			esac; \
		done; \
	done; \
	printf 'GENERATED FILE.  DO NOT EDIT.\n\n' 1>$(DIST_DIR)/ChangeLog; \
	if [ -d .git ]; then \
		$(GIT) log --pretty=fuller --stat --color=never \
			1>$(DIST_DIR)/ChangeLog; \
		$(GIT) log -n 1 --pretty='format:%ci' \
			1>$(DIST_DIR)/.source-date; \
		find $(dist_dir) | xargs touch -h -d \
			"$$(cat $(DIST_DIR)/.source-date)"; \
	elif [ -f ChangeLog ]; then \
		cp ChangeLog .source-date $(DIST_DIR); \
	fi; \
	find $(DIST_DIR) | LC_ALL=C sort | $(TAR) --no-recursion -T - \
		--owner=0 --group=0 --numeric-owner -cf $(DIST_DIR).tar; \
	touch -h -d "$$(cat $(DIST_DIR)/.source-date)" $(DIST_DIR).tar; \
	rm -Rf $(DIST_DIR)

dist-gzip: distdir
	$(V_DIST)$(GZIP) -c $(COMP_LVL_GZIP) \
		0<$(DIST_DIR).tar 1>$(DIST_ARCHIVE)
	$(Q)$(DO_RM_DIST_DIR)rm -Rf $(DIST_DIR).tar

dist-bzip2: distdir
	$(V_DIST)$(BZIP2) -c $(COMP_LVL_BZIP2) \
		0<$(DIST_DIR).tar 1>$(DIST_ARCHIVE)
	$(Q)$(DO_RM_DIST_DIR)rm -Rf $(DIST_DIR).tar

dist-xz: distdir
	$(V_DIST)$(XZ) -c $(COMP_LVL_XZ) \
		0<$(DIST_DIR).tar 1>$(DIST_ARCHIVE)
	$(Q)$(DO_RM_DIST_DIR)rm -Rf $(DIST_DIR).tar

dist dist-all:
	$(Q)$(MAKE) DO_RM_DIST_DIR=': ' V=$(V) $(DIST_TARGETS)
	$(Q)$(DO_RM_DIST_DIR)rm -Rf $(DIST_DIR).tar

.scad.stl:
	$(V_OPENSCAD)$(OPENSCAD) -d $*.d~ -o $*.stl $<
	$(Q)awk -v "PWD=$${PWD}/" $(DEP_AWK_SCRIPT) $*.d~ 1>$*.d
	$(Q)rm -f $*.d~

.scad.d:
	$(V_OPENSCAD)$(OPENSCAD) -d $*.d~ -o $*.stl $<
	$(Q)awk -v "PWD=$${PWD}/" $(DEP_AWK_SCRIPT) $*.d~ 1>$*.d
	$(Q)rm -f $*.d~

.stl.gcode:
	$(V_SLIC3R)$(SLIC3R) --load $*.ini --output $*.gcode \
		--print-center $(BED_CENTER) $*.stl | sed 's/^/           /'
