/*
 * PCI Express Half-Mini Card to Full-Mini-only socket bracket tray
 *
 * Copyright (C) 2020  Libiquity LLC
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

use <pcie-mc-pin-bracket.scad>

trgh  =   2;  /* Trough depth (tolerance for bracket length) */
hor_t =   4;  /* Horizontal wall thickness */
ver_t =   2;  /* Vertical wall thickness */
row_l = 100;  /* Row length */
row_n =   3;  /* Number of rows */
row_s =   6;  /* Row spacing */

module _pcie_mc_pin_bracket_tray_polygon()
{
	 w = pcie_mc_pin_bracket_width();
	vw = pcie_mc_pin_bracket_void_length()
	   - pcie_mc_pin_bracket_bridge_fillet() * 2;
	vh = pcie_mc_pin_bracket_void_width() + trgh;
	 s = row_s;

	polygon(points = [
			[vw/ 2      , ver_t + vh],
			[vw/ 2      , ver_t     ],
			[ w/ 2 + s/2, ver_t     ],
			[ w/ 2 + s/2, 0         ],
			[ w/-2 - s/2, 0         ],
			[ w/-2 - s/2, ver_t     ],
			[vw/-2      , ver_t     ],
			[vw/-2      , ver_t + vh],
		]);
}

module pcie_mc_pin_bracket_tray()
{
	w = pcie_mc_pin_bracket_width() + row_s;
	h = pcie_mc_pin_bracket_void_width() + trgh + ver_t;

	translate(v = [0, 0, h/2]) difference() {
		cube(size = [w*row_n + hor_t*2, row_l + hor_t*2, h],
			center = true);
		cube(size = [w*row_n, row_l, h + 2], center = true);
	}

	for (x = [w * (row_n/2 - 0.5) * -1 : w : w * (row_n/2 - 0.5) + w/2]) {
		translate(v = [x, row_l/2, 0]) rotate(a = [90, 0, 0])
			linear_extrude(height = row_l)
				_pcie_mc_pin_bracket_tray_polygon();
	}
}

pcie_mc_pin_bracket_tray();
